@@ -9,8 +9,10 @@ typedef enum {
   integer_t,
   integer64_t,
   double_t,
-  date_t,
-  datetime_t,
+  date_int_t,
+  date_double_t,
+  datetime_int_t,
+  datetime_double_t,
   time_t,
   string_t,
   ustring_t,
